"=============================================================================
" FILE: assets_complete.vim
" AUTHOR:  alpaca taichou <alprhcp666@gmail.com>
" Last Modified: 2013-02-25
" License: MIT license  {{{
"     Permission is hereby granted, free of charge, to any person obtaining
"     a copy of this software and associated documentation files (the
"     "Software"), to deal in the Software without restriction, including
"     without limitation the rights to use, copy, modify, merge, publish,
"     distribute, sublicense, and/or sell copies of the Software, and to
"     permit persons to whom the Software is furnished to do so, subject to
"     the following conditions:
"
"     The above copyright notice and this permission notice shall be included
"     in all copies or substantial portions of the Software.
"
"     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
"     OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
"     MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
"     IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
"     CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
"     TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
"     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
" }}}
"=============================================================================

let s:save_cpo = &cpo
set cpo&vim

let s:source = {
      \ 'name' : 'assets_complete',
      \ 'kind' : 'ftplugin',
      \ 'filetypes' : { 'ruby' : 1, 'haml' : 1, 'eruby' : 1,
      \   'scss': 1, 'sass' : 1, 'yaml': 1, 'ruby.rspec' : 1 },
      \ }

" let s:cached_regexp = {}

function! s:source.initialize() "{{{
  call neocomplcache#set_completion_length(
        \ 'assets_complete', g:neocomplcache_auto_completion_start_length)
  call neocomplcache#util#set_default_dictionary(
        \ 'g:neocomplcache_source_rank',
        \ 'assets_complete', 3)

  call alpaca_complete#let_g:('alpaca_complete_assets_dir', {
        \ 'img'   : 'app/assets/images',
        \ 'js'    : 'app/assets/javascripts',
        \ 'style' : 'app/assets/stylesheets',
        \})

  call alpaca_complete#let_g:('alpaca_complete_trigger_method', {
        \ 'img'   : 'asset_path,asset_data_uri,image-url,image-path,image_tag',
        \ 'js'    : 'javascript_include_tag',
        \ 'style' : 'stylesheet_link_tag',
        \})

  " Initialize.
  call alpaca_complete#initialize()
endfunction"}}}

function! s:source.finalize() "{{{
endfunction"}}}

function! s:get_real_assets_path(type) "{{{
  let type_path = g:alpaca_complete_assets_dir[a:type]

  return alpaca_complete#rails_root_join(type_path)
endfunction"}}}

function! s:source.get_keyword_pos(cur_text) "{{{
  " TODO assetsを使うfunctionに限定する
  " Filename pattern.
  let pattern = neocomplcache#get_keyword_pattern_end('filename')
  let [cur_keyword_pos, cur_keyword_str] =
        \ neocomplcache#match_word(a:cur_text, pattern)

  if cur_keyword_str =~ '//' ||
        \ (neocomplcache#is_auto_complete() &&
        \   cur_keyword_str =~# '\*\|\.\.\+$\|/c\%[ygdrive/]$')
    " Not Rails Project or Not filename pattern.
    return -1
  endif

  if neocomplcache#is_sources_complete() && cur_keyword_pos < 0
    let cur_keyword_pos = len(a:cur_text)
  endif

  return cur_keyword_pos
endfunction"}}}

function! s:source.get_complete_words(cur_keyword_pos, cur_keyword_str) "{{{
  let result = []

  for name in keys( g:alpaca_complete_assets_dir )
     let result += s:get_glob_files(a:cur_keyword_str, name)
  endfor

  return result
endfunction"}}}

" baseを基準としたパス以下のファイルから、
" cur_keyword_strにマッチするパスのみを抜き出す
function! s:get_glob_files(cur_keyword_str, type) "{{{
  let base = <SID>get_real_assets_path(a:type)
  if base !~ '/$'
    let base = base . '/'
  endif

  let image_list = neocomplcache#sources#filename_complete#get_complete_words(
        \ base . a:cur_keyword_str,
        \ base )

  let result = []
  for path in image_list
    let relative_path = substitute(path.word, base, '', 'g')
    let abbr = isdirectory(path.word) ? relative_path . '/' : relative_path

    let dict = {
          \ 'word': relative_path,
          \ 'abbr': abbr,
          \ 'menu' : '[ALP_' .toupper(a:type). ']'
          \ }

    call add(result, dict)
  endfor

  return result
endfunction"}}}

function! neocomplcache#sources#assets_complete#define() "{{{
  return s:source
endfunction"}}}

let &cpo = s:save_cpo
unlet s:save_cpo

" vim: foldmethod=marker
