"=============================================================================
" FILE: locale_complete.Vim
" AUTHOR:  Hiroyuki Ishii <alprhcp666@gmail.com>
" Last Modified: 2012-12-12
" License: MIT license  {{{
"     Permission is hereby granted, free of charge, to any person obtaining
"     a copy of this software and associated documentation files (the
"     "Software"), to deal in the Software without restriction, including
"     without limitation the rights to use, copy, modify, merge, publish,
"     distribute, sublicense, and/or sell copies of the Software, and to
"     permit persons to whom the Software is furnished to do so, subject to
"     the following conditions:
"
"     The above copyright notice and this permission notice shall be included
"     in all copies or substantial portions of the Software.
"
"     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
"     OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
"     MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
"     IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
"     CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
"     TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
"     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
" }}}
"=============================================================================

if exists('g:loaded_alpaca_complete') || &cp || v:version < 700
  finish
endif

let s:active = executable('alpaca_complete')
if !s:active "{{{
  echohl ErrorMsg
  echo 'requires gem ins alpaca_complete'
  echohl None
  finish
endif "}}}

let g:loaded_alpaca_complete = 1

let s:save_cpo = &cpo
set cpo&vim

" Initialize "{{{
let g:alpaca_complete_caching_list = {}
let s:source = {
      \ 'name' : 'locale_complete',
      \ 'kind' : 'ftplugin',
      \ 'filetypes' : { 'ruby' : 1, 'haml' : 1, 'eruby' : 1 },
      \}

if !exists('g:source_locale_complete_length')
  let g:source_locale_complete_length = 1
endif

if !exists('g:source_locale_complete_rank')
  let g:source_locale_complete_rank = 200
endif

if !executable('alpaca_complete')
  echohl ErrorMsg
  echo 'requires gem ins alpaca_complete'
  echohl None
  if executable('gem')
    call system('gem ins alpaca_complete')
  endif
  finish
endif
"}}}

function! s:source.initialize() "{{{
  if !s:active |return -1|endif
  call neocomplcache#set_completion_length('locale_complete', g:source_locale_complete_length)
  call neocomplcache#set_dictionary_helper(g:neocomplcache_source_rank,
        \ 'locale_complete', g:source_locale_complete_rank)

  " Initialize.
  call alpaca_complete#initialize()
endfunction "}}}

function! s:source.finalize() " {{{
endfunction " }}}

function! s:source.get_keyword_pos(cur_text) "{{{
  if !s:active |return -1|endif
  if exists('b:rails_root') && a:cur_text =~ '.*I18n\.t\A.*\|\(\A\|^\)t\A'
    let cur_pos  = match( a:cur_text, '[a-zA-Z]\+[a-zA-Z0-9._:-]*$' )

    return cur_pos < 0 ? -1 : cur_pos
  end

  return -1
endfunction "}}}

function! s:source.get_complete_words(cur_keyword_pos, cur_keyword_str) "{{{
  if !s:active |return -1|endif
  if !exists('b:rails_root')|return|endif
  let line = a:cur_keyword_str

  let input_first  = match(line, '[a-zA-Z._:]*$')
  let input_obj    = line[input_first :]

  if input_obj ==# '' || input_obj ==# "\s"
    return []
  endif

  let cache_name = s:get_cache_name()
  if !has_key(g:alpaca_complete_caching_list, cache_name)
    " load cache
    let keyword_lists = neocomplcache#cache#index_load_from_cache(
          \ 'alpaca_complete', cache_name)
    if !empty(keyword_lists)
      let g:alpaca_complete_caching_list[cache_name] = keyword_lists

      " Caching from cache.
      return neocomplcache#dictionary_filter(keyword_lists, input_obj)
    else
      call neocomplcache#sources#locale_complete#check_recache()
      return []
    endif
  else
    " use caching
    return neocomplcache#dictionary_filter(
          \ g:alpaca_complete_caching_list[cache_name] , input_obj)
  endif
endfunction "}}}

function! neocomplcache#sources#locale_complete#check_recache() "{{{
  if !s:active |return -1|endif
  let list_key = s:get_cache_name()
  " if !list_key |return -1| endif

  if !has_key(g:alpaca_complete_caching_list, list_key)
    call neocomplcache#sources#locale_complete#caching()
  endif
endfunction "}}}

function! s:get_rails_directory_name() "{{{
  if !exists('b:rails_root')|return ''|endif
  return b:rails_root[match(b:rails_root, '\/[^/]\+$') + 1:]
endfunction "}}}

function! s:get_cache_name() "{{{
  if !exists('b:rails_root')|return -1|endif
  return "locale_" . b:rails_root . "_locale"
endfunction"}}}

function! neocomplcache#sources#locale_complete#get_cur_text() "{{{
  return [line('.')]
endfunction "}}}

function! neocomplcache#sources#locale_complete#define() "{{{
  return s:source
endfunction"}}}

" caching
function! neocomplcache#sources#locale_complete#caching() "{{{
  if !exists('b:rails_root')|return -1|endif

  " RoRの補完プラグインalpaca_completeを使う
  let mf0     = alpaca_complete#system('locale', expand("%:p") )
  let mf_list = split(mf0, '\n')

  let result  = []
  let key_str   = ':key =>'
  let value_str = ':value =>'
  let key_len = strlen(key_str)
  let value_len = strlen(key_str)

  for locale_name in mf_list
    let key_end     = match( locale_name, value_str) - 2 " ' :'という文字列分引く
    let value_start = key_end + value_len + 2 + 2

    let locale      = {}
    let locale.key  = locale_name[key_len : key_end]
    "=> ja.error.messages
    let locale.type = substitute(locale.key, '^\([a-za-z_]*\)\..*', '\1', 'g')
    "=> ja en
    let locale.value= locale_name[value_start :]
    "=> ログイン情報を入力してください

    let result         = add(result, {
          \ "word" : locale.key[strlen(locale.type) + 1 :],
          \ "kind" : "[alpl_" . locale.type . "]",
          \ "menu" : locale.value,
          \})
  endfor

  if empty( result )
    return -1
  endif

  " caching
  call neocomplcache#cache#save_cache('alpaca_complete', s:get_cache_name(), result)
endfunction
"}}}

let &cpo = s:save_cpo
unlet s:save_cpo

" vimfoldmethod=marker
