# coding: utf-8


module AlpacaComplete::LocaleComplete
  LOCALE_PATH = 'config/locales'

  class << self
    # return array of list to match word
    def complete( path, word )
      opt = { "path" => path }
      comp = Regexp.new("^#{word}")

      get_locale_hash( opt ).select {|v| v.match comp }
    end

    # return array of locale list.
    def get_locale_list( opt={} )
      locales = get_locale_hash(opt)

      res = []
      locales.each do |k, v|
        res << k
      end

      res
    end

    # return hash of locale list. hash key is full_name.
    def get_locale_hash( opt={} )
      get_locale( opt["path"] ).join_keys(".")
    end

    # return hash deep_merged config/locales/**/*.yml
    def get_locale( file_path )
      require 'yaml'

      rails_root  = ::AlpacaComplete::Detect.detect( file_path )
      locale_path = "#{rails_root}/#{LOCALE_PATH}"

      # hash
      locales = Hash.new

      Dir.glob( "#{locale_path}/**/*.yml" ).each do |file|
        locale_hash = YAML::load_file( file )
        locales.deep_merge!(locale_hash)
      end

      locales
    end
  end

end
