# coding: utf-8

require 'optparse'

module AlpacaComplete::Utils
  class << self
    def option_parser( args )# {{{
      args << '-h' if args.empty?

      options = {}
      optparse = OptionParser.new do |opts|
        opts.banner  = get_banner

        opts.on('-h','--help','HELP') do
          puts opts
          exit
        end

        opts.on('-t CMD', '--type CMD', 'command type') do |l|
          options[:command] = l.to_sym
        end

        opts.on('-w VAL', '--word VAL', 'uncomplete word') do |l|
          options[:word] = l
        end

        opts.on('-f VAL', '--filepath VAL', 'rails path') do |l|
          options[:path] = l
        end

        opts.on('-v', '--version', 'Version') do
          puts ::AlpacaComplete::VERSION
          exit
        end
      end

      optparse.parse! args
      options[:files]  = args.to_a

      {
        command: options[:command],
        path: options[:path],
        word: options[:word],
        uncomplete: options[:locale]
      }
    end# }}}

    def get_banner# {{{
      return <<-BANNAR.gsub(/^\s*/)
          Usage:
          alpacaComplete [OPTIONS]
          ---------------------------------------------------------------------
      BANNAR
    end# }}}

    def run( cmd, path, uncomplete="" )# {{{
      case cmd
      when :locale
        # TODO :権限を委譲する
        locales = ::AlpacaComplete::LocaleComplete.complete( path, uncomplete )

        locales.each do |k, v|
          puts ":key =>#{k} :value =>#{v}"
        end

      when :assets_image
        assets = ::AlpacaComplete::AssetsComplete.complete( path, uncomplete, :image )
        assets.each { |file_path| puts file_path }

      # when :assets_stylesheets
      # when :assets_javascripts
      end
    end# }}}

    def load_yaml( path )# {{{
      return nil until File.exist?(path)

      require 'yaml'
      ::YAML::load_file( path )
    end# }}}
  end
end
